@@ -27,6 +27,7 @@ import signal
 import sys
 import time
 
+import enum
 from oslo_utils import importutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
@@ -62,26 +63,33 @@ class UnknownArgumentError(Exception):
 class ProcessExecutionError(Exception):
     def __init__(self, stdout=None, stderr=None, exit_code=None, cmd=None,
                  description=None):
+        super(ProcessExecutionError, self).__init__(
+            stdout, stderr, exit_code, cmd, description)
         self.exit_code = exit_code
         self.stderr = stderr
         self.stdout = stdout
         self.cmd = cmd
         self.description = description
 
+    def __str__(self):
+        description = self.description
         if description is None:
             description = _("Unexpected error while running command.")
+
+        exit_code = self.exit_code
         if exit_code is None:
             exit_code = '-'
+
         message = _('%(description)s\n'
                     'Command: %(cmd)s\n'
                     'Exit code: %(exit_code)s\n'
                     'Stdout: %(stdout)r\n'
                     'Stderr: %(stderr)r') % {'description': description,
-                                             'cmd': cmd,
+                                             'cmd': self.cmd,
                                              'exit_code': exit_code,
-                                             'stdout': stdout,
-                                             'stderr': stderr}
-        super(ProcessExecutionError, self).__init__(message)
+                                             'stdout': self.stdout,
+                                             'stderr': self.stderr}
+        return message
 
 
 class NoRootWrapSpecified(Exception):
@@ -97,8 +105,27 @@ def _subprocess_setup(on_preexec_fn):
         on_preexec_fn()
 
 
-LOG_ALL_ERRORS = 1
-LOG_FINAL_ERROR = 2
+@enum.unique
+class LogErrors(enum.IntEnum):
+    """Enumerations that affect if stdout and stderr are logged on error.
+
+    .. versionadded:: 2.7
+    """
+
+    #: No logging on errors.
+    DEFAULT = 0
+
+    #: Log an error on **each** occurence of an error.
+    ALL = 1
+
+    #: Log an error on the last attempt that errored **only**.
+    FINAL = 2
+
+
+# Retain these aliases for a number of releases...
+LOG_ALL_ERRORS = LogErrors.ALL
+LOG_FINAL_ERROR = LogErrors.FINAL
+LOG_DEFAULT_ERROR = LogErrors.DEFAULT
 
 
 class ProcessLimits(object):
@@ -107,36 +134,16 @@ class ProcessLimits(object):
     Attributes:
 
     * address_space: Address space limit in bytes
-    * core_file_size: Core file size limit in bytes
-    * cpu_time: CPU time limit in seconds
-    * data_size: Data size limit in bytes
-    * file_size: File size limit in bytes
-    * memory_locked: Locked memory limit in bytes
-    * number_files: Maximum number of open files
-    * number_processes: Maximum number of processes
+    * number_files: Maximum number of open files.
     * resident_set_size: Maximum Resident Set Size (RSS) in bytes
-    * stack_size: Stack size limit in bytes
 
     This object can be used for the *prlimit* parameter of :func:`execute`.
     """
 
-    _LIMITS = {
-        "address_space": "--as",
-        "core_file_size": "--core",
-        "cpu_time": "--cpu",
-        "data_size": "--data",
-        "file_size": "--fsize",
-        "memory_locked": "--memlock",
-        "number_files": "--nofile",
-        "number_processes": "--nproc",
-        "resident_set_size": "--rss",
-        "stack_size": "--stack",
-    }
-
     def __init__(self, **kw):
-        for limit in self._LIMITS.keys():
-            setattr(self, limit, kw.pop(limit, None))
-
+        self.address_space = kw.pop('address_space', None)
+        self.number_files = kw.pop('number_files', None)
+        self.resident_set_size = kw.pop('resident_set_size', None)
         if kw:
             raise ValueError("invalid limits: %s"
                              % ', '.join(sorted(kw.keys())))
@@ -144,10 +151,12 @@ class ProcessLimits(object):
     def prlimit_args(self):
         """Create a list of arguments for the prlimit command line."""
         args = []
-        for limit in self._LIMITS.keys():
-            val = getattr(self, limit)
-            if val is not None:
-                args.append("%s=%s" % (self._LIMITS[limit], val))
+        if self.address_space:
+            args.append('--as=%s' % self.address_space)
+        if self.number_files:
+            args.append('--nofile=%s' % self.number_files)
+        if self.resident_set_size:
+            args.append('--rss=%s' % self.resident_set_size)
         return args
 
 
@@ -188,17 +197,16 @@ def execute(*cmd, **kwargs):
     :param loglevel:        log level for execute commands.
     :type loglevel:         int.  (Should be logging.DEBUG or logging.INFO)
     :param log_errors:      Should stdout and stderr be logged on error?
-                            Possible values are None=default,
-                            LOG_FINAL_ERROR, or LOG_ALL_ERRORS. None
-                            implies no logging on errors. The values
-                            LOG_FINAL_ERROR and LOG_ALL_ERRORS are
-                            relevant when multiple attempts of command
-                            execution are requested using the
-                            'attempts' parameter. If LOG_FINAL_ERROR
-                            is specified then only log an error on the
-                            last attempt, and LOG_ALL_ERRORS requires
-                            logging on each occurence of an error.
-    :type log_errors:       integer.
+                            Possible values are
+                            :py:attr:`~.LogErrors.DEFAULT`,
+                            :py:attr:`~.LogErrors.FINAL`, or
+                            :py:attr:`~.LogErrors.ALL`. Note that the
+                            values :py:attr:`~.LogErrors.FINAL` and
+                            :py:attr:`~.LogErrors.ALL`
+                            are **only** relevant when multiple attempts of
+                            command execution are requested using the
+                            ``attempts`` parameter.
+    :type log_errors:       :py:class:`~.LogErrors`
     :param binary:          On Python 3, return stdout and stderr as bytes if
                             binary is True, as Unicode otherwise.
     :type binary:           boolean
@@ -232,6 +240,23 @@ def execute(*cmd, **kwargs):
     The *prlimit* parameter can be used to set resource limits on the child
     process.  If this parameter is used, the child process will be spawned by a
     wrapper process which will set limits before spawning the command.
+
+    .. versionchanged:: 3.4
+       Added *prlimit* optional parameter.
+
+    .. versionchanged:: 1.5
+       Added *cwd* optional parameter.
+
+    .. versionchanged:: 1.9
+       Added *binary* optional parameter. On Python 3, *stdout* and *stdout*
+       are now returned as Unicode strings by default, or bytes if *binary* is
+       true.
+
+    .. versionchanged:: 2.1
+       Added *on_execute* and *on_completion* optional parameters.
+
+    .. versionchanged:: 2.3
+       Added *preexec_fn* optional parameter.
     """
 
     cwd = kwargs.pop('cwd', None)
@@ -246,6 +271,8 @@ def execute(*cmd, **kwargs):
     shell = kwargs.pop('shell', False)
     loglevel = kwargs.pop('loglevel', logging.DEBUG)
     log_errors = kwargs.pop('log_errors', None)
+    if log_errors is None:
+        log_errors = LogErrors.DEFAULT
     binary = kwargs.pop('binary', False)
     on_execute = kwargs.pop('on_execute', None)
     on_completion = kwargs.pop('on_completion', None)
@@ -261,7 +288,9 @@ def execute(*cmd, **kwargs):
     if kwargs:
         raise UnknownArgumentError(_('Got unknown keyword args: %r') % kwargs)
 
-    if log_errors not in [None, LOG_ALL_ERRORS, LOG_FINAL_ERROR]:
+    if isinstance(log_errors, six.integer_types):
+        log_errors = LogErrors(log_errors)
+    if not isinstance(log_errors, LogErrors):
         raise InvalidArgumentError(_('Got invalid arg log_errors: %r') %
                                    log_errors)
 
@@ -420,6 +449,11 @@ def trycmd(*args, **kwargs):
 def ssh_execute(ssh, cmd, process_input=None,
                 addl_env=None, check_exit_code=True,
                 binary=False):
+    """Run a command through SSH.
+
+    .. versionchanged:: 1.9
+       Added *binary* optional parameter.
+    """
     sanitized_cmd = strutils.mask_password(cmd)
     LOG.debug('Running cmd (SSH): %s', sanitized_cmd)
     if addl_env:
