@@ -19,6 +19,7 @@ import errno
 import logging
 import multiprocessing
 import os
+import pickle
 import resource
 import stat
 import subprocess
@@ -48,6 +49,9 @@ exit 38"""
 # This byte sequence is undecodable from most encoding
 UNDECODABLE_BYTES = b'[a\x80\xe9\xff]'
 
+TRUE_UTILITY = (sys.platform.startswith('darwin') and
+                '/usr/bin/true' or '/bin/true')
+
 
 class UtilsTest(test_base.BaseTestCase):
     # NOTE(jkoelker) Moar tests from nova need to be ported. But they
@@ -71,11 +75,11 @@ class UtilsTest(test_base.BaseTestCase):
     def test_execute_with_callback(self):
         on_execute_callback = mock.Mock()
         on_completion_callback = mock.Mock()
-        processutils.execute("/bin/true")
+        processutils.execute(TRUE_UTILITY)
         self.assertEqual(0, on_execute_callback.call_count)
         self.assertEqual(0, on_completion_callback.call_count)
 
-        processutils.execute("/bin/true", on_execute=on_execute_callback,
+        processutils.execute(TRUE_UTILITY, on_execute=on_execute_callback,
                              on_completion=on_completion_callback)
         self.assertEqual(1, on_execute_callback.call_count)
         self.assertEqual(1, on_completion_callback.call_count)
@@ -92,7 +96,7 @@ class UtilsTest(test_base.BaseTestCase):
 
         self.assertRaises(IOError,
                           processutils.execute,
-                          "/bin/true",
+                          TRUE_UTILITY,
                           on_execute=on_execute_callback,
                           on_completion=on_completion_callback)
         self.assertEqual(1, on_execute_callback.call_count)
@@ -106,13 +110,13 @@ class UtilsTest(test_base.BaseTestCase):
         def preexec_fn():
             raise processutils.InvalidArgumentError()
 
-        processutils.execute("/bin/true")
+        processutils.execute(TRUE_UTILITY)
 
         expected_exception = (processutils.InvalidArgumentError if six.PY2
                               else subprocess.SubprocessError)
         self.assertRaises(expected_exception,
                           processutils.execute,
-                          "/bin/true",
+                          TRUE_UTILITY,
                           preexec_fn=preexec_fn)
 
 
@@ -319,7 +323,7 @@ grep foo
 
         self.assertEqual(attempts if attempts else 1, mock.mock.call_count)
         self.assertIn('Got an OSError', fixture.output)
-        self.assertIn('errno: 11', fixture.output)
+        self.assertIn('errno: %d' % errno.EAGAIN, fixture.output)
         self.assertIn("'/usr/bin/env false'", fixture.output)
 
     def test_logging_on_communicate_error_1(self):
@@ -464,6 +468,21 @@ grep foo
     def test_binary_undecodable_bytes_error(self):
         self.check_undecodable_bytes_error(True)
 
+    def test_picklable(self):
+        exc = processutils.ProcessExecutionError(
+            stdout='my stdout', stderr='my stderr',
+            exit_code=42, cmd='my cmd',
+            description='my description')
+        exc_message = str(exc)
+
+        exc = pickle.loads(pickle.dumps(exc))
+        self.assertEqual('my stdout', exc.stdout)
+        self.assertEqual('my stderr', exc.stderr)
+        self.assertEqual(42, exc.exit_code)
+        self.assertEqual('my cmd', exc.cmd)
+        self.assertEqual('my description', exc.description)
+        self.assertEqual(exc_message, str(exc))
+
 
 class ProcessExecutionErrorLoggingTest(test_base.BaseTestCase):
     def setUp(self):
@@ -733,7 +752,7 @@ class PrlimitTestCase(test_base.BaseTestCase):
         # Create a new soft limit for a resource, lower than the current
         # soft limit.
         soft_limit, hard_limit = resource.getrlimit(res)
-        if soft_limit <= 0:
+        if soft_limit < 0:
             soft_limit = default_limit
         else:
             soft_limit -= substract
@@ -771,31 +790,6 @@ class PrlimitTestCase(test_base.BaseTestCase):
         prlimit = self.limit_address_space()
         self.check_limit(prlimit, 'RLIMIT_AS', prlimit.address_space)
 
-    def test_core_size(self):
-        size = self.soft_limit(resource.RLIMIT_CORE, 1, 1024)
-        prlimit = processutils.ProcessLimits(core_file_size=size)
-        self.check_limit(prlimit, 'RLIMIT_CORE', prlimit.core_file_size)
-
-    def test_cpu_time(self):
-        time = self.soft_limit(resource.RLIMIT_CPU, 1, 1024)
-        prlimit = processutils.ProcessLimits(cpu_time=time)
-        self.check_limit(prlimit, 'RLIMIT_CPU', prlimit.cpu_time)
-
-    def test_data_size(self):
-        max_memory = self.memory_limit(resource.RLIMIT_DATA)
-        prlimit = processutils.ProcessLimits(data_size=max_memory)
-        self.check_limit(prlimit, 'RLIMIT_DATA', max_memory)
-
-    def test_file_size(self):
-        size = self.soft_limit(resource.RLIMIT_FSIZE, 1, 1024)
-        prlimit = processutils.ProcessLimits(file_size=size)
-        self.check_limit(prlimit, 'RLIMIT_FSIZE', prlimit.file_size)
-
-    def test_memory_locked(self):
-        max_memory = self.memory_limit(resource.RLIMIT_MEMLOCK)
-        prlimit = processutils.ProcessLimits(memory_locked=max_memory)
-        self.check_limit(prlimit, 'RLIMIT_MEMLOCK', max_memory)
-
     def test_resident_set_size(self):
         max_memory = self.memory_limit(resource.RLIMIT_RSS)
         prlimit = processutils.ProcessLimits(resident_set_size=max_memory)
@@ -806,16 +800,6 @@ class PrlimitTestCase(test_base.BaseTestCase):
         prlimit = processutils.ProcessLimits(number_files=nfiles)
         self.check_limit(prlimit, 'RLIMIT_NOFILE', nfiles)
 
-    def test_number_processes(self):
-        nprocs = self.soft_limit(resource.RLIMIT_NPROC, 1, 65535)
-        prlimit = processutils.ProcessLimits(number_processes=nprocs)
-        self.check_limit(prlimit, 'RLIMIT_NPROC', nprocs)
-
-    def test_stack_size(self):
-        max_memory = self.memory_limit(resource.RLIMIT_STACK)
-        prlimit = processutils.ProcessLimits(stack_size=max_memory)
-        self.check_limit(prlimit, 'RLIMIT_STACK', max_memory)
-
     def test_unsupported_prlimit(self):
         self.assertRaises(ValueError, processutils.ProcessLimits, xxx=33)
 
